@@ -510,6 +510,8 @@ sub _response_header_after_imp {
     # (Pragma...) which we don't know how to translate
     if ( defined $clen ) {
 	$DEBUG && $self->xdebug("have content-length $clen");
+    } elsif ( $self->{method} eq 'CONNECT' ) {
+	$DEBUG && $self->xdebug("have connect request");
     } else {
 	if  ( $version eq '1.1' and $self->{rq_version} eq '1.1' ) {
 	    $head->{'transfer-encoding'} = [ 'chunked' ];
@@ -548,8 +550,12 @@ sub _response_header_after_imp {
     $relay->forward(1,0,$hdr);
 
     if ( $self->{method} eq 'CONNECT' ) {
-	# upgrade server side and client side with SSL, but intercept traffic
-	$relay->sslify(1,0,$self->{rqhost});
+	# upgrade server side and client side with SSL, but intercept traffic.
+	# need to be called outside the current event handler, because $hdr
+	# will only be removed from rbuf after the current handler is done
+	App::HTTP_Proxy_IMP->once( sub {
+	    $relay->sslify(1,0,$self->{rqhost});
+	});
     }
 }
 
